	LIST      p=16F877            ; Dfinition de processeur
	#include <p16F877.inc>        ; fichier include
	#include <define.inc>        ; fichier include

;*****************************************************************************
;                             MACRO                                          *
;*****************************************************************************

BANK0	macro				; passer en banque0
		bcf	STATUS,RP0
		bcf	STATUS,RP1
	endm

BANK1	macro				; passer en banque1
		bsf	STATUS,RP0
		bcf	STATUS,RP1
	endm

BANK2	macro				; passer en banque2
		bcf	STATUS,RP0
		bsf	STATUS,RP1
	endm

BANK3	macro				; passer en banque3
		bsf	STATUS,RP0
		bsf	STATUS,RP1
	endm


;*****************************************************************************;
;				 Sauts inter-banques						;
;*****************************************************************************;

GOTOX	macro	ADRESSE			; saut inter-page
	local BIT4 = (ADRESSE & 0x1000)	; voir bit 12
	local BIT3 = (ADRESSE & 0x0800) ; voir bit 11
	local	ICI			; adresse courante
ICI
	local PICI = (ICI+2 & 0x1800)	; page du saut
	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	goto (ADRESSE & 0x7FF | PICI)	; adresse simule
	endm

GOTSX	macro	ADRESSE			; saut inter-page sans
					; slection de PCLATH
	local	ICI			; adresse courante
	local PICI = (ICI & 0x1800)	; page du saut
ICI
	goto (ADRESSE & 0x7FF | PICI)	; adresse simule
	endm


;*****************************************************************************;
;				 Sous-routines inter-banques					;
;*****************************************************************************;

CALLX	macro	ADRESSE			; call inter-page
	local BIT4 = (ADRESSE & 0x1000)	; voir bit 12
	local BIT3 = (ADRESSE & 0x0800) ; voir bit 11
	local	ICI			; adresse courante
ICI
	local PICI = (ICI+2 & 0x1800)	; page du saut
	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	call (ADRESSE & 0x7FF | PICI)	; adresse simule
	local BIT4 = ((ICI+5) & 0x1000)	; voir bit 12
	local BIT3 = ((ICI+5) & 0x0800)	; voir bit 11
	IF BIT3				; si page 1 ou 3
	bsf	PCLATH , 3		; b3 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 3		; b3 de PCLATH = 0
	ENDIF
	IF BIT4				; si page 2 ou 3
	bsf	PCLATH , 4		; b4 de PCLATH = 1
	ELSE				; sinon
	bcf	PCLATH , 4		; b4 de PCLATH = 0
	ENDIF
	endm

CALLSX	macro	ADRESSE			; sous-routine inter-page sans
					; slection de PCLATH
	local	ICI			; adresse courante
	local PICI = (ICI & 0x1800)	; page du saut
ICI
	call (ADRESSE & 0x7FF | PICI)	; adresse simule
	endm



;*****************************************************************************
;                      DEMARRAGE SUR RESET                                   *
;*****************************************************************************

	org 0x000 		; Adresse de dpart aprs reset

	goto	init		; ne rien faire


;*****************************************************************************
;                     ROUTINE INTERRUPTION                                   *
;*****************************************************************************
;-----------------------------------------------------------------------------
; Si on n'utilise pas l'adressage indirect dans les interrupts, on se passera
; de sauvegarder FSR
; Si le programme ne fait pas plus de 2K, on se passera de la gestion de 
; PCLATH
;-----------------------------------------------------------------------------
			;sauvegarder registres	
			;---------------------
	org 0x004		; adresse d'interruption

	BANK0			; passer en banque0
	
			; Interruption TMR0
			; -----------------

	btfsc	INTCON,T0IE		; tester si interrupt timer autorise
	btfss	INTCON,T0IF		; oui, tester si interrupt timer en cours
	goto 	intsw1		; non test suivant
	call	inttmr0		; oui, traiter interrupt tmr0
	bcf	INTCON,T0IF		; effacer flag interrupt tmr0

			; Interruption RB0/INT
			; --------------------
intsw1
	btfsc	INTCON,INTE		; tester si interrupt RB0 autorise
	btfss	INTCON,INTF		; oui, tester si interrupt RB0 en cours
	goto 	intsw2		; non sauter au test suivant
	call	intrb0		; oui, traiter interrupt RB0
	bcf	INTCON,INTF		; effacer flag interupt RB0

			; interruption RB4/RB7
			; --------------------
intsw2
	btfsc	INTCON,RBIE		; tester si interrupt RB4/7 autorise
	btfss	INTCON,RBIF		; oui, tester si interrupt RB4/7 en cours
	goto 	$+3			; non sauter
	call	intrb4		; oui, traiter interrupt RB4/7
	bcf	INTCON,RBIF		; effacer flag interupt RB4/7

	retfie  			; return from interrupt




;*****************************************************************************
;                          INITIALISATIONS                                   *
;*****************************************************************************

init

			; initialisation PORTS (banque 0 et 1)
			; ------------------------------------
	BANK0			; slectionner banque0
	clrf	PORTA		; Sorties PORTA  0
	clrf	PORTB		; sorties PORTB  0
	clrf	PORTC		; sorties PORTC  0
	clrf	PORTD		; sorties PORTD  0
	clrf	PORTE		; sorties PORTE  0
	bsf	STATUS,RP0	; passer en banque1
	movlw	ADCON1VAL	; PORTA en mode digital/analogique
	movwf	ADCON1	; criture dans contrle A/D
	movlw	DIRPORTA	; Direction PORTA
	movwf	TRISA		; criture dans registre direction
	movlw	DIRPORTB	; Direction PORTB
	movwf	TRISB		; criture dans registre direction
	movlw	DIRPORTC	; Direction PORTC
	movwf	TRISC		; criture dans registre direction
	movlw	DIRPORTD	; Direction PORTD
	movwf	TRISD		; criture dans registre direction
	movlw	DIRPORTE	; Direction PORTE
	movwf	TRISE		; criture dans registre direction

			; Registre d'options (banque 1)
			; -----------------------------
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; registres interruptions (banque 1)
			; ----------------------------------
	movlw	INTCONVAL	; charger valeur registre interruption
	movwf	INTCON	; initialiser interruptions
	movlw	PIE1VAL	; Initialiser registre 
	movwf	PIE1		; interruptions priphriques 1
	movlw	PIE2VAL	; initialiser registre
	movwf	PIE2		; interruptions priphriques 2

			; Effacer RAM banque 0
			; ---------------------
	BANK0			; slectionner banque0
	movlw	0x20		; initialisation pointeur
	movwf	FSR		; d'adressage indirect
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,7		; tester si fin zone atteinte (>7F)
	goto	init1		; non, boucler

			; autoriser interruptions (banque 0)
			; ----------------------------------
	clrf	PIR1		; effacer flags 1
	clrf	PIR2		; effacer flags 2
	bsf	INTCON,GIE	; valider interruptions
	goto	start		; programme principal


;*****************************************************************************
;                      PROGRAMME PRINCIPAL                                   *
;*****************************************************************************
start
	call debut


	include	"prog.asm"
	include	"variable.asm"
	include	"inter.asm"
	include	"tempo.asm"
	include	"lcd.asm"
	include	"menulcd.asm"
	include	"PWM.asm"
	include	"EEPROM.asm"
	include	"can.asm"
	include	"fonction.asm"
	include	"trame.asm"

	END 			; directive fin de programme

